
#include<util/twi.h>
#include<stdio.h>
#include<string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
//#include "lcd.h"

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//pt lcd





/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Bibliotec
 *********************************************************************************/
#define F_CPU 16000000
#include <util/delay.h>





/************************************************************************ 
 * DEFINE-uri pentru parametrizarea modulului LCD
 ************************************************************************/
#define LcdDATA_DDR		DDRC			// Portul pe care conectam firele de date la LCD-ul
#define LcdDATA_PORT	PORTC
#define LcdDATA_PIN		PINC

#define LcdCMD_DDR		DDRC			// Portul pe care conectam firele de comenzi la LCD
#define LcdCMD_PORT		PORTC
#define LcdCMD_PIN		PINC

#define LcdD4			 PC3				// Pin-ul pentru firul de date D4 de pe LCD
#define LcdD5			 PC4				// Pin-ul pentru firul de date D5 de pe LCD
#define LcdD6			 PC5				// Pin-ul pentru firul de date D6 de pe LCD
#define LcdD7			 PC6				// Pin-ul pentru firul de date D7 de pe LCD


#define LcdRS			 PC0			    // Pinul pentru selectare operatie (LCD)
#define LcdRW			 PC1				// Pinul pentru Read/ Write (LCD)
#define LcdE			 PC2				// Pinul de Enable (LCD)

#define RS_HIGH()				LcdCMD_PORT |= _BV(LcdRS)
#define RS_LOW()				LcdCMD_PORT &= ~_BV(LcdRS)
 
#define RW_HIGH()				LcdCMD_PORT |= _BV(LcdRW)
#define RW_LOW()				LcdCMD_PORT &= ~_BV(LcdRW)
 
#define ENABLE()				LcdCMD_PORT |= _BV(LcdE)
#define DISABLE()				LcdCMD_PORT &= ~_BV(LcdE)

#define LCD_INSTR_4wire 		0x28 	// 4 fire date, font 5x8
#define LCD_INSTR_display 		0x0C 	// Display On, Cursor On, Blinking On ( 1 Display Cursor Blink )
#define LCD_INSTR_clearDisplay  	0x01 	// Clear Display
#define LCD_INSTR_returnHome		0x02 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_nextLine 		0xC0 	// Return Cursor and LCD to Home Position
#define LCD_INSTR_gotoCGRAM		0x40	// go to Character Generator RAM

/*************
 * API LCD   *
 *************/
/*****************************
 * IMPLEMENTARE API LCD.     *
 *****************************/

void LCD_init()
{
	LcdDATA_DDR |=  (1<<LcdD4)|(1<<LcdD5)|(1<<LcdD6)|(1<<LcdD7);	// setam pinii de date ca pini de iesire
	LcdCMD_DDR  |=  (1<<LcdRS)|(1<<LcdRW)|(1<<LcdE);				// setam pinii de comenzi ca pini de iesire

	LCD_waitNotBusy();
	
	LcdCMD_PORT   &= ~(1<<LcdRS);									// Setam linia RS pe low
	LcdCMD_PORT   &= ~(1<<LcdRW);									// Setam linia RW pe low (acum suntem in modul de trimis instructiuni)
	LcdDATA_PORT  &= ~(1<<LcdD4)&~(1<<LcdD6)&~(1<<LcdD7); 		// Specificam ca vrem 4 fire de date, prima comanda (LcdD5 activ, restul nu)
	LcdDATA_PORT  |=  (1<<LcdD5);									// setam pinii de comenzi ca pini de iesire
	
	LcdCMD_PORT |=  (1<<LcdE);					// Setam linia E(nable) pe high; aceasta ii specifica LCD-ului sa preia datele
	LCD_waitInstructions(12);					// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE );				// Setam linia E(nable) pe low; transferul s-a terminat

	LCD_writeInstruction(LCD_INSTR_4wire); 		// Incarcam comanda: 4 bit data, 2 lines, 5x8 font
	LCD_writeInstruction(LCD_INSTR_display); 	// Display On, Cursor On, Blinking On
	LCD_writeInstruction(0x06);					// Increment, no shift
	LCD_writeInstruction(0x01);					// Clear Display

	
}

void LCD_writeInstruction(unsigned char _instruction)
{			
	
	LCD_waitNotBusy(); 							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	RS_LOW();									// setam pinul RS pe low (low=instructiuni, high=date)
	RW_LOW();									// setam pinul RW pe low (suntem in modul de comenzi acum)
	LCD_write(_instruction);					// apelam procedura ce trimite byte-ul pe firele de date
}


void LCD_writeData(unsigned char _data)
{
	LCD_waitNotBusy(); 							// asteptam ca LCD-ul sa fie liber sa primeasca comenzile	
	RS_HIGH();									// setam pinul RS pe high (low=instructiuni, high=date)
	RW_LOW();									// setam pinul RW pe low (suntem in modul de date acum)
	LCD_write(_data);							// apelam procedura ce trimite byte-ul pe firele de date

}

void LCD_write(unsigned char _byte)
{
	unsigned char _byte2;
	
	_byte2 = _byte>>4;

	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte2, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
		
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte2, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte2, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte2, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	
	LcdDATA_PORT &= ~(1<<LcdD4);
	if ( bit_is_set( _byte, 0 ) )
		LcdDATA_PORT |= (1<<LcdD4);
	
	LcdDATA_PORT &= ~(1<<LcdD5);
	if ( bit_is_set( _byte, 1 ) )
		LcdDATA_PORT |= (1<<LcdD5);

	LcdDATA_PORT &= ~(1<<LcdD6);
	if ( bit_is_set( _byte, 2 ) )
		LcdDATA_PORT |= (1<<LcdD6);

	LcdDATA_PORT &= ~(1<<LcdD7);
	if ( bit_is_set( _byte, 3 ) )
		LcdDATA_PORT |= (1<<LcdD7);

	LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
	LCD_waitInstructions(12);						// Asteptam o perioada de timp T
	LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
	LCD_waitInstructions(12);
}

void LCD_waitNotBusy()
{
_delay_ms(10);

	unsigned char _loop = 1;

	while (_loop)
	{
		LcdDATA_DDR &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7);	// Setam pinii de date de la LCD pe in pt a citi busy flag
		LcdDATA_PORT &= ~(1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); 	// Dezactivam pullup resistor pentru pinii de in
		
		LcdCMD_PORT &= ~(1<<LcdE);					// Setam pin-ul e pe low; ar trebui sa fie deja low, doar ne asiguram
		LcdCMD_PORT &= ~(1<<LcdRS);					// Setam pinul RS pe low
		LcdCMD_PORT |=  (1<<LcdRW);					// Setam pinul RW pe high (acum suntem in modul de interogare busy/adr)
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		_loop = LcdDATA_PIN & (1<<LcdD7);				// Citim busy flag-ul
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low
		
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		
		LcdCMD_PORT |= (1<<LcdE);						// Setam Pinul E pe high
		LCD_waitInstructions(12);						// Asteptam o perioada de timp T
		LcdCMD_PORT &= ~(1<<LcdE);						// Setam Pinul E pe low

		LcdDATA_DDR |= (1<<LcdD4 | 1<<LcdD5 | 1<<LcdD6 | 1<<LcdD7); // Setam Portul de LCD ca port de iesire la loc
	}
}

void LCD_waitInstructions(unsigned char _instructions)
{
	while (_instructions--)
		;
}


void LCD_printDecimal2u(unsigned int _n)
{
	unsigned char tmp=0;
	
	// Extragem sutele
	while(_n>=100)
		_n-=100;

	while(_n>=10){
		tmp++;
		_n-=10;
	}

	LCD_writeData(tmp+'0');
	LCD_writeData(_n+'0');
}

void LCD_printHexa(unsigned int _n)
{
	unsigned char _tmp = _n>>4;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
	_tmp = _n & 0x0F;
	if (_tmp>9)
		_tmp += 'A'-10;
	else
		_tmp += '0';
	LCD_writeData( _tmp );
}


void LCD_print(char* _msg)
{
	unsigned char i=0;
	for( ; _msg[i]!=0 && i<16; i++)
		LCD_writeData( _msg[i] );
}

void LCD_print2(char* _msg1, char* _msg2)
{
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print(_msg1);
	LCD_writeInstruction(LCD_INSTR_nextLine);
	LCD_print(_msg2);
}


void lcdInit4Wires(void) 
{
	DDRC = 0b11111111;
 	LCD_waitNotBusy();
	PORTC = 0b00000010 | (1<<PC6);
	asm volatile("nop \n\t nop \n\t nop \n\t nop \n\t nop \n\t nop \n\t");
	PORTC = 0b00000010 ;
	LCD_writeInstruction(0b00101000);
	LCD_writeInstruction(0b00001111);
	LCD_writeInstruction(0b00000110);
	LCD_writeInstruction(0b00000001);
}

void lcdClearscreen(void)
{
	LCD_writeInstruction(0b00000001);	
}


void setFirstRow(){LCD_writeInstruction(0b10000000);}
void setSecondRow(){LCD_writeInstruction(0b11000000);}






























////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

char zcode[5];
char ycode[5];
char   str[5];
int alarm=0;
int senzor=0;
int nrsecunde=0;


void initcode()
{
	int i;
	for(i=0;i<4;i++)
	{
		zcode[i]='0'+i;
		ycode[i]='9'-i;
	}
	zcode[4]='\0';
	ycode[4]='\0';
}
void newzcode()
{
	int i;
	for(i=0;i<4;i++)
		zcode[i]=str[i];
	zcode[4]='\0';
	
}
void initports()

{
	DDRA=0xFF;
    PORTA=0x00;
    DDRB = 0xFF;
	PORTB = 0x00;
	DDRC = 0xFF;
    PORTC = 0xFF;
	DDRD = 0xFF;
	PORTD = 0xFF;
	
}
void delay(long int x)
{
	while(x!=0){
		asm volatile("nop");
		x--;
	}
	return;
}
void bigdelay()
{
	int i,j;
	for(i=0;i<1000;i++)
		for(j=0;j<5000;j++)
				asm volatile("nop");
	return;
}

char readkeyboard()
{
	char k='%';
	unsigned char tmp=0, tmp1=0,tmp2=0,tmp3=0;
	
	PORTA = 0xFF;
	PORTA&=~(1<<4);
	delay(5);
	tmp=PINA;
	
	
	if (tmp==0xE7) { 	k='A';	LCD_writeData(k);asm volatile("nop");delay(500000);} //for tmp == 0b11100111
	if (tmp==0xEB) { 	k='3';	LCD_writeData(k);asm volatile("nop");delay(500000);}//for tmp = 0b11101011
	if (tmp==0xED) { 	k='2';	LCD_writeData(k);asm volatile("nop");delay(500000);} //for tmp = 0b11101101
	if (tmp==0xEE) { 	k='1';	LCD_writeData(k);asm volatile("nop");delay(500000);} //for tmp = 0b11101110
	
	
	PORTA = 0xFF;
	PORTA&=~(1<<5);

	delay(5);
	
	tmp1=PINA;
	
	if (tmp1==0xDF) { 	k='B';	LCD_writeData(k);asm volatile("nop");delay(500000);} //for tmp1 == 0b11011111
	if (tmp1==0xDB) { 	k='6';	LCD_writeData(k);asm volatile("nop");delay(500000);} //for tmp1 == 0b11011011
	if (tmp1==0xDD) { 	k='5';	LCD_writeData(k);asm volatile("nop");delay(500000);} //for tmp1 == 0b11011101
	if (tmp1==0xDE) { 	k='4';	LCD_writeData(k);asm volatile("nop");delay(500000);} //for tmp1 == 0b11011110 
		
	PORTA = 0xFF;
	PORTA&=~(1<<6);
	
	delay(5);
	
	tmp2=PINA;
	
	if (tmp2==0xB7) { 	k='C';	LCD_writeData(k);asm volatile("nop");delay(500000);}//tmp2==0b10110111
	if (tmp2==0xBB) { 	k='9';	LCD_writeData(k);asm volatile("nop");delay(500000);}//tmp2==0b10111011
	if (tmp2==0xBD) { 	k='8';	LCD_writeData(k);asm volatile("nop");delay(500000);}//tmp2==0b10111101
	if (tmp2==0xBE) { 	k='7';	LCD_writeData(k);asm volatile("nop");delay(500000);}//tmp2==0b10111110
	
	
	PORTA = 0xFF;
	PORTA&=~(1<<7);
	delay(5);
	
	tmp3=PINA;
	if (tmp3==0x77) { 	k='\0';	LCD_INSTR_clearDisplay;asm volatile("nop");delay(500000);}//for tmp3==0b01110111
	if (tmp3==0x7B) { 	k='#';	LCD_writeData(k);asm volatile("nop");delay(500000);}//for tmp3==0b01111011
	if (tmp3==0x7D) { 	k='0';	LCD_writeData(k);asm volatile("nop");delay(500000);}//for tmp3==0b01111101
	if (tmp3==0x7E) { 	k='*';	LCD_writeData(k);asm volatile("nop");delay(500000);}//for tmp3==0b01111110)
	
	return k;
}
int comparezcode()
{
	int i;
	for(i=0;i<5;i++)
		if(str[i]!=zcode[i]) return 0;
	return 1;
}
int compareycode()
{
	int i;
	for(i=0;i<5;i++)
		if(str[i]!=ycode[i]) return 0;
	return 1;
}

void armalarm()
{
	if (alarm == 0) 
		alarm = 1;
	else
		alarm = 0;
	
}
void showStatus()
{
	lcdClearscreen();
	switch(alarm){
		case -2:{	
					LCD_print("Enter new code: ");
				}break;
		case -1:{
					LCD_print("Enter arm code: ");
				}break;
		case 0:{
					/**nearmata**/
					LCD_print("Unarmed!!! Code:");
					PORTB=0xFF;
					PORTB&=~(1<<3);
			   }break;
		case 1:{
					LCD_print("Armed!!! Code:  ");
					PORTB=0xFF;
					PORTB&=~(1<<3);
			   }break;
		case 2:{
					//un senzor a fost activat
					LCD_print("Senzor ");
		            LCD_writeData('1'+senzor);
		            LCD_print(" trig.");
		            setSecondRow();
		            PORTB = 0xFF;
		            PORTB&=~(1<<3);
			   }break;
		case 3:{
					//Alarmaaaaaaaaaaaaaa!!!
					LCD_print("Senzor ");
		            LCD_writeData('1'+senzor);
		            LCD_print(" Alarrrrrrrrrrrrrrrm");
		            setSecondRow();
		            PORTB = 0xFF;
			   }
	}
}
void inittimer1()
{
	nrsecunde=0;
	OCR1A=15625; //prescaler 1024 -> 16k/1024
	TCCR1A=0;
	TCCR1B=0b00001101;	//prescaler 1024
	TIMSK=16;		//interupt timer1_compa
	sei();
	
}
void stoptimer1()
{
	OCR1A=0;
	TCCR1A=0;
	TCCR1B=0;
	TIMSK=0;
	cli();
	
}

ISR (TIMER1_COMPA_vect)
{
	if (alarm==0) stoptimer1();	//daca alarma este nearmata nu are nici un sens sa tin timerul in functiune
	nrsecunde++; //numar secundele
	if (nrsecunde>15) //daca s-a depasit numarul de secunde asteptat  opresc timnerul si intru in mod alarmaaa!!! 
	{
		stoptimer1();
		alarm=3;
		showStatus();
	}
	
}

int checkStatus(){
	if (alarm>-1 && comparezcode()==1) {armalarm();return 0;} //din nearmata devine armata 
	if (alarm==0 && compareycode()==1) {alarm=-1;return 0;}//din starea neramata in starea de introduce a codului care ma anunta de schimbare
	if (alarm==-1 && comparezcode()==1) {alarm=-2;return 0;} //daca s-a introdus codul vechi de armare ma lasa sa introduc noul cod de armare
	if (alarm==-2 && compareycode()!=1) {alarm=0;LCD_INSTR_clearDisplay;newzcode();LCD_print("New code accepted!");bigdelay();return 0;}
	return 1;
}
void initSenzor(int aux){
	LCD_INSTR_clearDisplay;alarm=2; senzor=aux;inittimer1();
}

int main(void)
{
	initcode();
	initports();
	LCD_INSTR_4wire;
	char s;
	char senz;
	
	int j=0;
	while(1)
	{
		showStatus();
		while (1)
		{
			s=readkeyboard();
			if(s!='%')
			{
				str[j++]=s;
				if (str[j-1]=='\0')	
				{
					if (j!=11) {j=0;break;} //daca nu am introdus exact 10 caractere atunci se cere reintroducerea codului
					if(checkStatus() == 0)
					{j = 0;break;}
					break;
				}
			}
			senz=PINB;
			if (alarm==1) //daca este armata astept sa vad ce senzor se activeaza
			{
				
				if ((senz & 2)==0) {initSenzor(1);break;}
				
			}
		}
	}
	return 0;
}
